@@ -27,48 +27,40 @@ SOFTWARE.
 from .items import Items
 
 
-class SkyBlock:
+class SkyBlockUtils:
     """ Utilities relating to Hypixel SkyBlock """
     @staticmethod
     def getItem(item: str, *, reverse: bool = False) -> str:
         r"""Gets an item name from an item ID.
 
-        Parameters
-        -----------
-        item :class:`str`
-            The item ID.
+        :param item: The item ID.
+        :type item: str
 
-        reverse :class:`Optional[bool]`
-            Whether or not to translate an item name to an item ID.
+        :param reverse: Whether or not to translate an item name to an item ID.
+        :type reverse: Optional[bool]
+
+        :return: The item name/ID.
+        :rtype: str"""
 
-        Returns
-        --------
-        :class:`str`
-            The item name/ID."""
         if not reverse:
             if item not in Items.ids:
                 return item
             return Items.ids[item]
-        else:
-            for id_ in Items.ids:
-                if Items.ids[id_] == item:
-                    return id_
-            return item
+        for id_ in Items.ids:
+            if Items.ids[id_] == item:
+                return id_
+        return item
 
     @staticmethod
     def getMinionSlots(crafted: list) -> int:
         r"""Gets the number of crafted minion slots a player has.
 
-        Parameters
-        -----------
-        crafted :class:`list`
-            The player's crafted minions.
+        :param crafted: The player's crafted minions.
+        :type crafted: list
+
+        :return: The player's crafted minion slots.
+        :rtype: int"""
 
-        Returns
-        --------
-        :class:`int`
-            The player's crafted minion slots."""
-        slts = 5
         req_unique = [5,
                       15,
                       30,
@@ -93,27 +85,19 @@ class SkyBlock:
             if minion not in minions:
                 minions.append(minion)
         rexp = len(minions)
-        for level in req_unique:
-            if rexp >= level:
-                slts += 1
-            elif rexp < level:
-                break
-        slots = slts
+        slots = SkyBlockUtils.getLevel(req_unique, rexp, subtract=False)+5
         return slots
 
     @staticmethod
     def zombieSlayer(xp) -> int:
         r"""Gets the level for the Zombie slayer from the slayer experience.
 
-        Parameters
-        -----------
-        xp :class:`float`
-            The player's Zombie slayer XP.
+        :param xp: The player's Zombie slayer XP.
+        :type xp: float
+
+        :return: The player's Zombie slayer level.
+        :rtype: int"""
 
-        Returns
-        --------
-        :class:`int`
-            The player's Zombie slayer level."""
         req_xp = [5,
                   15,
                   200,
@@ -123,27 +107,19 @@ class SkyBlock:
                   100000,
                   400000,
                   1000000]
-        lvl = 0
-        for level in req_xp:
-            if xp >= level:
-                lvl += 1
-            elif xp < level:
-                break
+        lvl = SkyBlockUtils.getLevel(req_xp, xp, subtract=False)
         return lvl
 
     @staticmethod
     def spiderSlayer(xp) -> int:
         r"""Gets the level for the Spider slayer from the slayer experience.
 
-        Parameters
-        -----------
-        xp :class:`float`
-            The player's Wolf slayer XP.
+        :param xp: The player's Spider slayer XP.
+        :type xp: float
+
+        :return: The player's Spider slayer level.
+        :rtype: int"""
 
-        Returns
-        --------
-        :class:`int`
-            The player's Spider slayer level."""
         req_xp = [5,
                   25,
                   200,
@@ -153,27 +129,18 @@ class SkyBlock:
                   100000,
                   400000,
                   1000000]
-        lvl = 0
-        for level in req_xp:
-            if xp >= level:
-                lvl += 1
-            elif xp < level:
-                break
+        lvl = SkyBlockUtils.getLevel(req_xp, xp, subtract=False)
         return lvl
 
     @staticmethod
     def wolfSlayer(xp: float) -> int:
         r"""Gets the level for the Wolf slayer from the slayer experience.
 
-        Parameters
-        -----------
-        xp :class:`float`
-            The player's Wolf slayer XP.
+        :param xp: The player's Wolf slayer XP.
+        :type xp: float
 
-        Returns
-        --------
-        :class:`int`
-            The player's Wolf slayer level."""
+        :return: The player's Wolf slayer level.
+        :rtype: int"""
         req_xp = [10,
                   30,
                   250,
@@ -183,54 +150,43 @@ class SkyBlock:
                   100000,
                   400000,
                   1000000]
-        lvl = 0
-        for level in req_xp:
-            if xp >= level:
-                lvl += 1
-            elif xp < level:
-                break
+        lvl = SkyBlockUtils.getLevel(req_xp, xp, subtract=False)
         return lvl
 
     @staticmethod
     def slayerLevels(data: dict) -> (int, int, int):
-        r""" Retrieves a SkyBlock player's Slayer levels from their profile data.
+        r"""Retrieves a SkyBlock player's Slayer levels from their profile data.
 
-        Parameters
-        -----------
-        data :class`dict`
-            The player's profile data.
+        :param data: The player's profile data.
+        :type data: dict
+
+        :return: A tuple with the player's Slayer levels in the order Zombie, Spider, and Wolf.
+        :retype: int, int, int"""
 
-        Returns
-        --------
-        :class:`(int, int, int)`
-            A tuple with the player's Slayer levels in the order Zombie, Spider, and Wolf."""
         try:
-            zombie = SkyBlock.zombieSlayer(data['slayer_bosses']['zombie']['xp'])
+            zombie = SkyBlockUtils.zombieSlayer(data['slayer_bosses']['zombie']['xp'])
         except KeyError:
             zombie = 0
         try:
-            spider = SkyBlock.spiderSlayer(data['slayer_bosses']['spider']['xp'])
+            spider = SkyBlockUtils.spiderSlayer(data['slayer_bosses']['spider']['xp'])
         except KeyError:
             spider = 0
         try:
-            wolf = SkyBlock.wolfSlayer(data['slayer_bosses']['wolf']['xp'])
+            wolf = SkyBlockUtils.wolfSlayer(data['slayer_bosses']['wolf']['xp'])
         except KeyError:
             wolf = 0
         return zombie, spider, wolf
 
     @staticmethod
     def getSkillLevel(xp: float) -> int:
-        r""" Converts skill XP to a skill level.
+        r"""Converts skill XP to a skill level.
 
-        Parameters
-        -----------
-        xp: :class:`float`
-            The skill experience.
+        :param xp: The skill experience.
+        :type xp: float
+
+        :return: The skill's level.
+        :rtype: int"""
 
-        Returns
-        --------
-        :class:`int`
-            The skill's level."""
         prog_req_xp = [50,
                        125,
                        200,
@@ -281,33 +237,21 @@ class SkyBlock:
                        3400000,
                        3700000,
                        4000000]
-        lvl = 0
-        rexp = xp
-        for level in prog_req_xp:
-            if rexp >= level:
-                rexp -= level
-                lvl += 1
-                pass
-            elif rexp < level:
-                break
+        lvl = SkyBlockUtils.getLevel(prog_req_xp, xp)
         return lvl
 
     @staticmethod
     def getRuneCraftLevel(xp: float) -> int:
-        r""" Converts runecrafting skill XP to a skill level.
+        r"""Converts runecrafting skill XP to a skill level.
 
-        This is seperate from :function:`getSkillLevel` because runecrafting has
+        This is seperate from py:function:`getSkillLevel` because runecrafting has
         different experience requirements.
 
-        Parameters
-        -----------
-        xp: :class:`float`
-            The runecrafting skill experience.
+        :param xp: The runecrafting skill experience.
+        :type xp: float
 
-        Returns
-        --------
-        :class:`int`
-            The runecrafting skill's level."""
+        :return: The runecrafting skill's level.
+        :rtype: int"""
         prog_req_xp = [50,
                        100,
                        125,
@@ -332,30 +276,18 @@ class SkyBlock:
                        9800,
                        12200,
                        15300]
-        lvl = 0
-        rexp = xp
-        for level in prog_req_xp:
-            if rexp >= level:
-                rexp -= level
-                lvl += 1
-                pass
-            elif rexp < level:
-                break
+        lvl = SkyBlockUtils.getLevel(prog_req_xp, xp)
         return lvl
 
     @staticmethod
     def farmingCollection(data: dict) -> dict:
-        r""" Gets a player's SkyBlock Farming collection from their member data.
+        r"""Gets a player's SkyBlock Farming collection from their member data.
 
-        Parameters
-        -----------
-        data :class:`dict`
-            The player's SkyBlock data.
+        :param data: The player's SkyBlock data.
+        :type data: dict
 
-        Returns
-        --------
-        :class:`dict`
-            A dict of their items in the Farming collection."""
+        :return: A dict of their items in the Farming collection.
+        :rtype: dict"""
         collections = ['WHEAT',
                        'CARROT_ITEM',
                        'POTATO_ITEM',
@@ -538,41 +470,18 @@ class SkyBlock:
                           100000]
             else:
                 req_xp = []
-            if len(req_xp) != 0:
-                col = SkyBlock.getItem(collection)
-                if collection in data['collection']:
-                    next_lvl = 0
-                    amount = data['collection'][collection]
-                    lvl = 0
-                    for level in req_xp:
-                        if amount >= level:
-                            lvl += 1
-                            pass
-                        elif amount < level:
-                            next_lvl = level
-                            break
-                    if lvl >= len(req_xp):
-                        c[col] = lvl
-                    else:
-                        progress = int(round(amount, 0))/next_lvl
-                        c[col] = lvl + progress
-                else:
-                    c[col] = 0
+            c = SkyBlockUtils.getCollectionData(data, req_xp, collection, c)
         return c
 
     @staticmethod
     def miningCollection(data: dict):
-        r""" Gets a player's SkyBlock Mining collection from their member data.
+        r"""Gets a player's SkyBlock Mining collection from their member data.
 
-        Parameters
-        -----------
-        data :class:`dict`
-            The player's SkyBlock data.
+        :param data: The player's SkyBlock data.
+        :type data: dict
 
-        Returns
-        --------
-        :class:`dict`
-            A dict of their items in the Mining collection."""
+        :return: A dict of their items in the Mining collection.
+        :rtype: dict"""
         collections = ['COBBLESTONE',
                        'COAL',
                        'IRON_INGOT',
@@ -695,41 +604,18 @@ class SkyBlock:
                           50000]
             else:
                 req_xp = []
-            if len(req_xp) != 0:
-                col = SkyBlock.getItem(collection)
-                if collection in data['collection']:
-                    next_lvl = 0
-                    amount = data['collection'][collection]
-                    lvl = 0
-                    for level in req_xp:
-                        if amount >= level:
-                            lvl += 1
-                            pass
-                        elif amount < level:
-                            next_lvl = level
-                            break
-                    if lvl >= len(req_xp):
-                        c[col] = lvl
-                    else:
-                        progress = int(round(amount, 0)) / next_lvl
-                        c[col] = lvl + progress
-                else:
-                    c[col] = 0
-            return c
+            c = SkyBlockUtils.getCollectionData(data, req_xp, collection, c)
+        return c
 
     @staticmethod
     def combatCollection(data: dict) -> dict:
-        r""" Gets a player's SkyBlock Combat collection from their member data.
+        r"""Gets a player's SkyBlock Combat collection from their member data.
 
-        Parameters
-        -----------
-        data :class:`dict`
-            The player's SkyBlock data.
+        :param data: The player's SkyBlock data.
+        :type data: dict
 
-        Returns
-        --------
-        :class:`dict`
-            A dict of their items in the Combat collection."""
+        :return: A dict of their items in the Combat collection.
+        :rtype: dict"""
         collections = ['ROTTEN_FLESH',
                        'BONE',
                        'STRING',
@@ -795,41 +681,18 @@ class SkyBlock:
                           50000]
             else:
                 req_xp = []
-            if len(req_xp) != 0:
-                col = SkyBlock.getItem(collection)
-                if collection in data['collection']:
-                    next_lvl = 0
-                    amount = data['collection'][collection]
-                    lvl = 0
-                    for level in req_xp:
-                        if amount >= level:
-                            lvl += 1
-                            pass
-                        elif amount < level:
-                            next_lvl = level
-                            break
-                    if lvl >= len(req_xp):
-                        c[col] = lvl
-                    else:
-                        progress = int(round(amount, 0)) / next_lvl
-                        c[col] = lvl + progress
-                else:
-                    c[col] = 0
-            return c
+            c = SkyBlockUtils.getCollectionData(data, req_xp, collection, c)
+        return c
 
     @staticmethod
     def foragingCollection(data: dict) -> dict:
-        r""" Gets a player's SkyBlock Foraging collection from their member data.
+        r"""Gets a player's SkyBlock Foraging collection from their member data.
 
-        Parameters
-        -----------
-        data :class:`dict`
-            The player's SkyBlock data.
+        :param data: The player's SkyBlock data.
+        :type data: dict
 
-        Returns
-        --------
-        :class:`dict`
-            A dict of their items in the Foraging collection."""
+        :return: A dict of their items in the Foraging collection.
+        :rtype: dict"""
         collections = ['LOG',
                        'LOG:1',
                        'LOG:2',
@@ -900,41 +763,18 @@ class SkyBlock:
                           25000]
             else:
                 req_xp = []
-            if len(req_xp) != 0:
-                col = SkyBlock.getItem(collection)
-                if collection in data['collection']:
-                    next_lvl = 0
-                    amount = data['collection'][collection]
-                    lvl = 0
-                    for level in req_xp:
-                        if amount >= level:
-                            lvl += 1
-                            pass
-                        elif amount < level:
-                            next_lvl = level
-                            break
-                    if lvl >= len(req_xp):
-                        c[col] = lvl
-                    else:
-                        progress = int(round(amount, 0)) / next_lvl
-                        c[col] = lvl + progress
-                else:
-                    c[col] = 0
-            return c
+            c = SkyBlockUtils.getCollectionData(data, req_xp, collection, c)
+        return c
 
     @staticmethod
     def fishingCollection(data: dict):
-        r""" Gets a player's SkyBlock Fishing collection from their member data.
+        r"""Gets a player's SkyBlock Fishing collection from their member data.
 
-        Parameters
-        -----------
-        data :class:`dict`
-            The player's SkyBlock data.
+        :param data: The player's SkyBlock data.
+        :type data: dict
 
-        Returns
-        --------
-        :class:`dict`
-            A dict of their items in the Fishing collection."""
+        :return: A dict of their items in the Mining collection.
+        :rtype: dict"""
         collections = ['RAW_FISH',
                        'RAW_FISH:1',
                        'RAW_FISH:2',
@@ -1039,24 +879,69 @@ class SkyBlock:
                           4000]
             else:
                 req_xp = []
-            if len(req_xp) != 0:
-                col = SkyBlock.getItem(collection)
-                if collection in data['collection']:
-                    next_lvl = 0
-                    amount = data['collection'][collection]
-                    lvl = 0
-                    for level in req_xp:
-                        if amount >= level:
-                            lvl += 1
-                            pass
-                        elif amount < level:
-                            next_lvl = level
-                            break
-                    if lvl >= len(req_xp):
-                        c[col] = lvl
-                    else:
-                        progress = int(round(amount, 0)) / next_lvl
-                        c[col] = lvl + progress
+            c = SkyBlockUtils.getCollectionData(data, req_xp, collection, c)
+        return c
+
+    @staticmethod
+    def getCollectionData(data, req_xp, collection, c):
+        """Modifies a collection's collection data.
+
+        :param data: The raw API data.
+        :type data: dict
+
+        :param req_xp: A list of the required XP.
+        :type req_xp: list
+
+        :param collection: The collection to modify the dict with.
+        :type collection: str
+
+        :param c: The dict to modify.
+        :type c: dict
+
+        :return: The modified dict.
+        :rtype: dict"""
+        if len(req_xp) != 0:
+            col = SkyBlockUtils.getItem(collection)
+            if collection in data['collection']:
+                next_lvl = 0
+                amount = data['collection'][collection]
+                lvl = 0
+                for level in req_xp:
+                    if amount >= level:
+                        lvl += 1
+                    elif amount < level:
+                        next_lvl = level
+                        break
+                if lvl >= len(req_xp):
+                    c[col] = lvl
                 else:
-                    c[col] = 0
-            return c
+                    progress = int(round(amount, 0)) / next_lvl
+                    c[col] = lvl + progress
+            else:
+                c[col] = 0
+        return c
+
+    @staticmethod
+    def getLevel(req_xp, xp, *, subtract=True):
+        """Gets a level from required xp and total xp.
+
+        :param req_xp: The required XP.
+        :type req_xp: list
+
+        :param xp: The total XP.
+        :type xp: float
+
+        :param subtract: Whether or not to subtract from xp. Defaults to ``True``
+        :type subtract: bool
+
+        :return: The level.
+        :rtype: int"""
+        lvl = 0
+        for level in req_xp:
+            if xp >= level:
+                if subtract:
+                    xp -= level
+                lvl += 1
+            elif xp < level:
+                break
+        return lvl
